#include "UserAPI.h"

/*
 * NOTE: You are actually writing a Palm program even if you make no 
 * modifications to your easyC code!  Things like strcpy either don't 
 * exist or work differently than you are used to.  Get a good Palm
 * programming book if you need to know more than this :-)
 */
#include <PalmOS.h>  

/* all this is all optional up to the "Program 1" comment.  The code in the 
 * programs is mostly bogus so trash it when you have something real to put 
 * there.
 */

#define false 0
#define true !false
#define min(a, b) (((a) < (b)) ? (a) : (b))

#define DEBUG 0

#define DANGER 5
#define SLOWDOWN 20
#define OPENUP 30
#define SPEEDUP  60
#define INFINITY 99

#define SMALL 260
#define BIG 400 

#define MOTOR_L 1
#define MOTOR_R 2
#define ULTRA_B_INT 1
#define ULTRA_B_DATA 11
#define ULTRA_L_INT 2
#define ULTRA_L_DATA 12
#define ULTRA_F_INT 3
#define ULTRA_F_DATA 13
#define ULTRA_R_INT 4
#define ULTRA_R_DATA 14
#define BUMPER_L 9
#define BUMPER_R 10
#define ENCODER_L 5
#define ENCODER_R 6

/* ----- The motor on the Right is oriented to standard
 * Left - Right
 * 0    - 255
 * 255  - 0
 * B=255,0 F=0,255 R=0,0 L=255,255
 */

char speed = 127;
int last_turn = -1;
int turn_count = 0;
 
void stop( void )
{ 
	speed = 0;
	SetMotor(MOTOR_L, 127);
	SetMotor(MOTOR_R, 127);
}

void forward(int _speed)
{
	if (_speed > 127) _speed = 127;
	if (_speed < -127) _speed = -127;
	speed = _speed;
	
	PrintToScreen("F(%d) \n", speed);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, 127-speed);
	SetMotor(MOTOR_R, 127+speed);
	Wait(50+speed);
}

void back(void)
{
	PrintToScreen("B(%d) ", speed);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, 127+speed);
	SetMotor(MOTOR_R, 127-speed);
	Wait(100+speed);
}

void left(int waiting)
{
	if (last_turn == 1)
	{
		if (++turn_count > 9) turn_count = 1;
	}
	else
		turn_count = 0;
	last_turn = 0;
	
	PrintToScreen("L(%d %d %d) \n", speed, last_turn, turn_count);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, 127+speed);
	SetMotor(MOTOR_R, 127+speed);
	if (turn_count > 0)
		waiting+=20*turn_count;
	Wait(waiting);
	forward(speed);
}

void right(int waiting)
{
	if (last_turn == 0)
	{
		if (++turn_count > 9) turn_count = 1;
	}
	else
		turn_count = 0;
	last_turn = 1;
	
	PrintToScreen("R(%d %d %d) \n", speed, last_turn, turn_count);
	if (DEBUG) return;
	
	SetMotor(MOTOR_L, 127-speed);
	SetMotor(MOTOR_R, 127-speed);
	if (turn_count > 0)
		waiting+=10*turn_count;
	Wait(waiting);
	forward(speed);
}

/*
 * Speed = 0..127
 * Turn Left == curve<0,  Turn Right == curve>0
 * returns bumper that was bumped or zero.
 */
unsigned char move(int speed, int curve, int duration)
{
	int left, right;
	unsigned char bumps[3] = { BUMPER_L, BUMPER_R, 0 };
	
	if (speed > 127)  speed = 127;
	if (speed < -127) speed = -127;
	if (curve > 127)  curve = 127;
	if (curve < -127) curve = -127;
	left = 127-speed-curve;
	if (left < 0) left = 0;
	if (left > 254) left = 254;
	right = 127+speed-curve;
	if (right < 0) right = 0;
	if (right > 254) right = 254;
	
	//PrintToScreen("M(%d,%d:%d) \n", speed, curve, duration);
	if (DEBUG) return 0;
	
	SetMotor(MOTOR_L, left);
	SetMotor(MOTOR_R, right);
	return WaitB(duration, bumps);
}

void back_left(void)
{
	move(-75,-20,70);
	move(50, 0, 0);
}

void back_right(void)
{
	move(-75,20,70);
	move(50, 0, 0);
}

int bumpers( void )
{
	if (GetDigitalInput(BUMPER_L) == 0)
	{
		PrintToScreen("Bumped<\n");
		back_right();
		return true;
	}
	else if (GetDigitalInput(BUMPER_R) == 0)
	{
		PrintToScreen("Bumped>\n");
		back_left();
		return true;
	}
	
	return false;
}

//------------------------------------------------------------------------------
// Program 1
//------------------------------------------------------------------------------

char *prog1init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Rover");
	return name;
}

int prog1 ( void )
{
	int uL = 1;
	int uR = 2;
	int uM = 3;
	int diff;
	int iters = -1;
	
	if (DEBUG != 1) move(127,0,0);
	while ( iters && VexRunning() )
	{
		bumpers();
		uM = ReadUltrasonic(ULTRA_F_INT, ULTRA_F_DATA);
		if (uM < SLOWDOWN)
		{
			if (uM < DANGER)
			{
				move(-75, 0, 70);
				move(0, 0, 0);
			}
			else
				move(50+uM, 0, 0);
			uL = ReadUltrasonic(ULTRA_L_INT, ULTRA_L_DATA);
			uR = ReadUltrasonic(ULTRA_R_INT, ULTRA_R_DATA);
			diff = uR-uL;
			if (diff>0 && diff<20) diff = 20;
			if (diff<0 && diff>-20) diff = -20;
			if (min(uL, uR) < SLOWDOWN)
				move(-75, diff, 50);
			else 
				move(50+min(uL,uR), diff, 0);
			
		}
		/* --- doing nothing isn't right...
		else if (uM == INFINITY)
		{
			// Infinities can be spurious readings from oblique angles to a wall.
			// Need to check side sonics to see if we should turn away from it.
		}
		*/
		else
		{
			if (bumpers()) continue;
			uL = ReadUltrasonic(ULTRA_L_INT, ULTRA_L_DATA);
			if (uM == INFINITY && uL == INFINITY) continue;
			if (bumpers()) continue;
			uR = ReadUltrasonic(ULTRA_R_INT, ULTRA_R_DATA);
			if (uM == INFINITY && uR == INFINITY) continue;
			if (bumpers()) continue;
			diff = uR-uL;
			move(50+uM, diff, 0);
		}
		
		
		/*
		// If uL or uR == 99 it may be a wall at an angle rather than a 
		// large open space.  Treat this differently by examining the 
		// value of the other sensor first.
		diff = uL - uR;
		if (uL < DANGER)
			back_right();
		else if (uR < DANGER)
			back_left();
		if (diff > OPENUP)
			move(speed,-50,1);
		else if (diff < -OPENUP)
			move(speed,50,1);
		else if (uL < SLOWDOWN || uR < SLOWDOWN)
		{
			turn_count = 0;
			last_turn = -1;
			move(50+min(uL,uR),-diff,1);
			//forward(50);
		}
		else if (uL > SPEEDUP && uR > SPEEDUP)
		{
			turn_count = 0;
			last_turn = -1;
			move(50+min(uL,uR),0,1);
			//forward(127);
		}
		else  // SLOWDOWN <= uL and uR <= SPEEDUP
		{
			turn_count = 0;
			last_turn = -1;
			move(50+min(uL,uR),0,1);
		}		
		*/
	}
	move(0, 0, 0); // Stop motors
	
	if (!iters) PrintToScreen("Program Stopped\n");
	
	return 0;
}

//------------------------------------------------------------------------------
// Program 2
//------------------------------------------------------------------------------

char *prog2init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Table");
	return name;
}

int prog2( void )
{
	unsigned long encoder, etest;
	unsigned char sound, test;
	int running = true;
	
	StartUltrasonic(ULTRA_F_INT, ULTRA_F_DATA);
	test = sound = GetUltrasonic(ULTRA_F_INT, ULTRA_F_DATA);
	PrintToScreen("s=%d\n", sound);
	PresetEncoder ( ENCODER_L, 0 ) ;
	StartEncoder ( ENCODER_L) ;
	move(35,-4 ,0);  // forward seems to pull to the right
	while (running && test<sound+5)
	{
		test = GetUltrasonic(ULTRA_F_INT, ULTRA_F_DATA);
		PrintToScreen("st=%d\n", test);
		running = VexRunning();
	}
	encoder = GetEncoder ( ENCODER_L) ; // Get stop value before actually stopped
	move(0,0,0);
	PrintToScreen("e=%ld\n", encoder);
	if (!running) return 0;
	MakeSound(1500, 200);
	
	PresetEncoder ( ENCODER_L, 0 ) ;
	etest = GetEncoder(ENCODER_L);
	PrintToScreen("et=%ld\n", etest);
	move(-35,-4,0);
	if (encoder < 8) encoder = 8;  // Prevent "infinite" loop w/unsigned negative number
	while (VexRunning() && etest < encoder-8) // fudge off the encoder value to compensate for slop in the readings
	{
		etest = GetEncoder(ENCODER_L);
		PrintToScreen("et=%ld\n", etest);
	}
	move(0,0,0);
	StopEncoder( ENCODER_L);
	
	return 0;
}

//------------------------------------------------------------------------------
// Program 3
//------------------------------------------------------------------------------

char *prog3init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Spiral");
	return name;
}

int prog3( void )
{
	int iters;
	int sound = 2900;
	
	if (DEBUG != 1) forward(70);
	iters = 127-70;
	while ( iters>0 && VexRunning() )
	{
		MakeSound(1000+iters*10, 90);
		move(50, iters, 300-3*iters);
		iters -= 5;
	}
	forward(0);
	return 0;

	// Alternate version w/o using move() function
	PrintToScreen("Spiral\n");

	SetMotor(MOTOR_R,200);
	for (iters=150; iters>100; iters-=5, sound-=75)
	{
		MakeSound(sound, 90);
		SetMotor(MOTOR_L, 50);
		Wait(30);
		SetMotor(MOTOR_L,iters);
		Wait(50);
	}
	SetMotor(MOTOR_L, 50);
	Wait(30);
	forward(0);
	return 0;
}

//------------------------------------------------------------------------------
// Program 4
//------------------------------------------------------------------------------

// North is the direction the robot is initially facing when placed in maze
#define North 0
#define South 1
#define East  2
#define West  3
#define HOME  4
#define Lost  5

void GetBias(int servo_bias, int direction)
{
	while (VexRunning() && GetDigitalInput(BUMPER_R) == 1)
	{
		if (GetDigitalInput(BUMPER_L) == 0)
		{
			servo_bias += direction;
			SetServo(3, 127-servo_bias);
			while (GetDigitalInput(BUMPER_L) == 1)
				Wait(10);
		}
	}
	PrintToScreen("Servo Bias = %d\n", servo_bias);
}

unsigned char backof(unsigned char direction)
{
	switch (direction)
	{
		case North: return South;
		case South: return North;
		case East:  return West;
		case West:  return East;
		default:	return Lost;
	}
}
	
unsigned char eastof(unsigned char direction)
{
	switch (direction)
	{
		case North: return East;
		case South: return West;
		case East:  return South;
		case West:  return North;
		default:	return Lost;
	}
}
	
unsigned char westof(unsigned char direction)
{
	switch (direction)
	{
		case North: return West;
		case South: return East;
		case East:  return North;
		case West:  return South;
		default:	return Lost;
	}
}
	
typedef struct _node {
	unsigned char home;
	struct _node *node[4];
	unsigned char sonic[4];
	unsigned char ticks[4];
} NODE;

NODE *newnode()
{
	unsigned char i;
	
	NODE *node = (NODE *) MemPtrNew(sizeof(NODE));
	
	node->home = Lost;
	
	for (i=0; i<4; i++)
	{
		node->node[i]   = NULL;
		node->sonic[i]  = Lost;
		node->ticks[i] = 0;
	}
	
	return node;
}

void ReadSonics(NODE *node, unsigned char direction)
{
		node->sonic[direction]         = ReadUltrasonic(ULTRA_F_INT, ULTRA_F_DATA);
		node->sonic[westof(direction)] = ReadUltrasonic(ULTRA_L_INT, ULTRA_L_DATA);
		node->sonic[eastof(direction)] = ReadUltrasonic(ULTRA_R_INT, ULTRA_R_DATA);
		node->sonic[backof(direction)] = ReadUltrasonic(ULTRA_B_INT, ULTRA_B_DATA);
		PrintToScreen("N=%d S=%d W=%d E=%d\n", node->sonic[North], node->sonic[South],
			node->sonic[West], node->sonic[East]);
}

NODE *retrace(unsigned char direction, NODE *from)
{
	PresetEncoder ( ENCODER_L, 0 ) ;
	move(50, 0, 0);
	while (VexRunning() && GetEncoder(ENCODER_L) < from->ticks[direction])
		Wait(10);
	
	return from->node[direction];	
}

NODE *go(unsigned char direction, NODE *from)
{
	NODE *next;
	
	if (from->ticks[direction] != 0) return retrace(direction, from);
	
	next = (NODE *) MemPtrNew(sizeof(NODE));
	
	next->home = backof(direction);
	next->node[backof(direction)] = from;
	
	PresetEncoder ( ENCODER_L, 0 ) ;
	move(50, 0, 0);
	while (VexRunning())
	{
		ReadSonics(next, direction);
		if (   GetDigitalInput(BUMPER_L) == 0
			|| GetDigitalInput(BUMPER_R) == 0
			|| next->sonic[direction] < 10
			|| next->sonic[westof(direction)] - 5 > from->sonic[westof(direction)]
			|| next->sonic[eastof(direction)] - 5 > from->sonic[eastof(direction)]
			)
		{
			from->ticks[direction] = 
				next->ticks[backof(direction)] = 
				GetEncoder(ENCODER_L);
			move(0, 0, 0);
			return next;
		}
	}
	
	return next;
}

unsigned char turn(NODE *from, NODE *to, unsigned char direction)
{
	unsigned char newdir;
	
	if (!to || to->sonic[direction] > 10) return direction;  // keep going straight if possible
	
	if (to->sonic[westof(direction)] <= 10 && to->sonic[eastof(direction)] <= 10)
	{
		PrintToScreen("Turn around\n");
		newdir = backof(direction);
		move(0, 50, 50);  // Need 180 deg turn here...
	}
	else if (to->sonic[westof(direction)]  > from->sonic[eastof(direction)])
	{
		PrintToScreen("Turn West\n");
		newdir = westof(direction);
		move(0, 50, 25);  // Need 90 deg turn here...
	}
	else
	{
		PrintToScreen("Turn East\n");
		newdir = eastof(direction);
		move(0, -50, 25);  // Need -90 deg turn here...
	}
		
	move(0, 0, 0);
	return newdir;
}

unsigned char *prog4init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Hydra");
	return name;
}

int prog4( void )
{
	char servo_bias = 60;
	unsigned char direction = North;
	NODE *root = newnode();
	NODE *last, *next;
	
	SetServo(3, 127-servo_bias);
	// GetBias(servo_bias, 1);
	
	last = root;
	ReadSonics(last, direction);
	last->home = HOME;
	next = go(direction, last);
	direction = turn(last, next, direction);
	while (VexRunning() && next && next->home != HOME)
	{
		last = next;
		next = go(direction, next);
		direction = turn(last, next, direction);
	}
	return 0;
}	
	
//------------------------------------------------------------------------------
// Program 5
//------------------------------------------------------------------------------

typedef struct _path {
	int x, y;
	unsigned char sonics[4];
} PATH;

// S2I is conversion factor from sonic values to inches
#define S2I 0.6923
void GetSonics(PATH *path, int heading)
{
		path->sonics[heading]         = ReadUltrasonic(ULTRA_F_INT, ULTRA_F_DATA) * S2I;
		path->sonics[backof(heading)] = ReadUltrasonic(ULTRA_B_INT, ULTRA_B_DATA) * S2I;
		path->sonics[westof(heading)] = ReadUltrasonic(ULTRA_L_INT, ULTRA_L_DATA) * S2I;
		path->sonics[eastof(heading)] = ReadUltrasonic(ULTRA_R_INT, ULTRA_R_DATA) * S2I;
}

int AdvanceInch(PATH *path, int heading)
{
	int go = 1;
	unsigned long e;
	
	PresetEncoder ( ENCODER_L, 0 ) ;
	e = 0;
	move(30, -4, 250);
	/*
	while (VexRunning() && e < 7 )
	{
		e = GetEncoder(ENCODER_L);
	}
	*/
	move(0, 0, 0);
	GetSonics(path, heading);
	e = GetEncoder(ENCODER_L);
	PrintToScreen("E=%ld, S=%d\n", e, path->sonics[heading]);
	
	MakeSound(1500, 200);
	
	if (GetDigitalInput(BUMPER_L) == 0) { go = 0;  }
	if (GetDigitalInput(BUMPER_R) == 0) { go = 0;  }
	
	return go;
}

unsigned long AdvanceFully(PATH *path, int heading, int *bumped)
{
	float target;
	unsigned long e, t;
	unsigned char bumps[3] = { BUMPER_L, BUMPER_R, 0 };
	
	*bumped = 0;
	
	// Convert inches to ticks
	target = (path->sonics[heading] - 8) * 10.4174;
	t = target;
	if (t < 0) t = 0;
	
	PresetEncoder ( ENCODER_L, 0 ) ;
	StartEncoder ( ENCODER_L) ;
	if (!move(50, -4, 5*t))  // move returns zero if no bumpers bumped
		while (VexRunning() && (e = GetEncoder(ENCODER_L)) < t)
		{
			if (WaitB(5*(t-e), bumps)) { *bumped = 1; break; }
		}
	else *bumped = 1;
	move(0, 0, 0);
	MakeSound(1500, 200);
	StopEncoder ( ENCODER_L) ;
	if (*bumped) MakeSound(1500, 200);
	
	e = GetEncoder(ENCODER_L);
	PrintToScreen("Sonic=%d Target=%ld Actual=%ld\n", path->sonics[heading], t, e);
	return e;
}

unsigned long AdvanceTo(PATH *path, int heading, int x, int y)
{
	unsigned long e, t;
	unsigned char bumps[3] = { BUMPER_L, BUMPER_R, 0 };
	
	switch (heading) {
		case North: { t = path->y - y; break; }
		case South: { t = y - path->y; break; }
		case East:  { t = path->x - x; break; }
		case West:  { t = x - path->x; break; }
		default: return 0;
	}
	
	if (t < 0) return 0;
	
	PresetEncoder ( ENCODER_L, 0 ) ;
	StartEncoder ( ENCODER_L) ;
	if (!move(50, -4, 10))  // move returns zero if no bumpers bumped
		while (VexRunning() && (e = GetEncoder(ENCODER_L)) < t)
		{
			if (WaitB(10, bumps)) break;
		}
	move(0, 0, 0);
	MakeSound(1500, 200);
	StopEncoder ( ENCODER_L) ;
	
	e = GetEncoder(ENCODER_L);
	PrintToScreen("Target=%ld Actual=%ld\n", t, e);
	return e;
}

int Turn(PATH *path, int heading)
{
	StartEncoder ( ENCODER_L) ;
	PresetEncoder ( ENCODER_L, 0 ) ;
	StartEncoder ( ENCODER_R) ;
	PresetEncoder ( ENCODER_R, 0 ) ;
	if (path->sonics[westof(heading)] > path->sonics[eastof(heading)])
	{
		heading = westof(heading);
		move(0, -50, 500);
	}
	else
	{
		heading = eastof(heading);
		move(0, 50, 500);
	}
	while (VexRunning() && GetEncoder(ENCODER_R) < 60)
	{
		// Tight loop
		Wait(1);
	}
	move(0, 0, 50);
	//PrintToScreen("Turn El=%d Er=%d\n", GetEncoder(ENCODER_L), GetEncoder(ENCODER_R));
	StopEncoder ( ENCODER_L) ;
	StopEncoder ( ENCODER_R) ;
	
	return heading;
}

int TurnTo(PATH *path, int heading, int x, int y)
{
	int turning = 0;
	
	StartEncoder ( ENCODER_L) ;
	PresetEncoder ( ENCODER_L, 0 ) ;
	StartEncoder ( ENCODER_R) ;
	PresetEncoder ( ENCODER_R, 0 ) ;
	//PrintToScreen("TurnTo from(%d, %d) to(%d, %d)\n", x, y, path->x, path->y);
	switch (heading) {
		case North: { turning = path->x - x; break; }
		case South: { turning = x - path->x; break; }
		case West:  { turning = path->y - y; break; }
		case East:  { turning = y - path->y; break; }
	}
	if (turning < 0) 
	{
		//PrintToScreen("TurnTo Left\n");
		heading = westof(heading);
		move(0, -50, 500);
	}
	else if (turning > 0)
	{
		//PrintToScreen("TurnTo Right\n");
		heading = eastof(heading);
		move(0, 50, 500);
	}
	else
		PrintToScreen("Where are we going?\n");
	while (VexRunning() && GetEncoder(ENCODER_R) < 60)
	{
		// Tight loop
		Wait(1);
	}
	move(0, 0, 50);
	//PrintToScreen("TurnTo El=%d Er=%d\n", GetEncoder(ENCODER_L), GetEncoder(ENCODER_R));
	StopEncoder ( ENCODER_L) ;
	StopEncoder ( ENCODER_R) ;
	
	return heading;
}

int Reverse(int heading)
{
	StartEncoder ( ENCODER_L) ;
	PresetEncoder ( ENCODER_L, 0 ) ;
	StartEncoder ( ENCODER_R) ;
	PresetEncoder ( ENCODER_R, 0 ) ;
	heading = backof(heading);
	move(0, 50, 500);
	while (VexRunning() && GetEncoder(ENCODER_R) < 130)
	{
		// Tight loop
		Wait(10);
	}
	move(0, 0, 50);
	//PrintToScreen("Reverse El=%d Er=%d\n", GetEncoder(ENCODER_L), GetEncoder(ENCODER_R));
	StopEncoder ( ENCODER_L) ;
	StopEncoder ( ENCODER_R) ;
	
	return heading;
}

void SetPosition(int heading, int *x, int *y, unsigned long d)
{
	PrintToScreen("(%d, %d) -> ", *x, *y);
	switch (heading) {
		case North: { *y += d; break; }
		case South: { *y -= d; break; }
		case East:  { *x += d; break; }
		case West:  { *x -= d; break; }
		default: break;
	}
	PrintToScreen("(%d, %d)\n", *x, *y);
}

void PrintHeading(int heading)
{
	switch (heading) {
		case North: { PrintToScreen("North\n"); break; }
		case South: { PrintToScreen("South\n"); break; }
		case East:  { PrintToScreen("East\n"); break; }
		case West:  { PrintToScreen("West\n"); break; }
		default: break;
	}
}

char *prog5init( void )
{
	char *name = (char *) MemPtrNew(10);
	StrCopy(name, "Mapper");
	return name;
}

/* 
 * Need to create progNstop() to deal with turning off motors and such
 * and have it called from palmvex.c when an exception is caught.
 */
 
int prog5( void)
{
	char servo_bias;
	int heading;
	int x=0, y=0;
	unsigned long d;
	
	PATH *path = (PATH *) MemPtrNew(sizeof(PATH)*100);
	int p = 0;
	int bumped = 0;
	
	// Set a starting point
	path[0].x = 0; path[0].y = 0;
	path[0].sonics[0] = 0;
	path[1].sonics[0] = 0;
	path[2].sonics[0] = 0;
	path[3].sonics[0] = 0;
	
	// Get sonics pointing on ordinals
	//heading = East;
	//servo_bias = -106;  // East Bias
	heading = North;
	servo_bias = 60;  // North Bias
	SetServo(3, 127-servo_bias);
	// GetBias(servo_bias, -1);
	
	// Get sonics in inches
	PrintHeading(heading);
	GetSonics(&path[p], heading);
	if (path[p].sonics[heading] < 8) 
	{
		PrintToScreen("Can't move %d %d %d %d\n", 
			path[p].sonics[heading],
			path[p].sonics[backof(heading)],
			path[p].sonics[westof(heading)],
			path[p].sonics[eastof(heading)]);
		return 0;
	}
	d = AdvanceFully(&path[p], heading, &bumped);
	SetPosition(heading, &x, &y, d);
	
	while (VexRunning() && !bumped && p<100)
	{
		if (path[p].sonics[heading] < 68)
		{
			p++;
			path[p].x = x;
			path[p].y = y;
			GetSonics(&path[p], heading);
			heading = Turn(&path[p], heading);
			PrintHeading(heading);
		}
		else
		{
			GetSonics(&path[p], heading);
		}
		
		if (path[p].sonics[heading] < 8) bumped = 1;
		else {
			d = AdvanceFully(&path[p], heading, &bumped);
			SetPosition(heading, &x, &y, d);
		}
	}

	// Turn 180 deg	 and go back
	PrintToScreen("Back up\n");
	move(-50, -4, 500);
	move(0, 0, 0);
	heading = Reverse(heading);
	PrintHeading(heading);
	//SetPosition(heading, &x, &y, 30);  // take off the amount backed up
	
	while (VexRunning() && p>=0)
	{
		d = AdvanceTo(&path[p], heading, x, y);
		//SetPosition(heading, &x, &y, d);
		x = path[p].x;
		y = path[p].y;
		if (--p>=0) {
			heading = TurnTo(&path[p], heading, x, y);
			PrintHeading(heading);
		}
	}
	
	heading = Reverse(heading);
	PrintHeading(heading);
	
	return 0;
}

